#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>

#include <linux/types.h>
#include <linux/videodev.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "fitsio.h"

#include <sys/time.h>
#include <time.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#define VIDEODEVICE "/dev/video0"

#define WIDTH 640
#define HEIGHT 480
#define WIDTH2 640
#define HEIGHT2 480
#define DISPSCALE 1
#define NPIX 307200
#define WAITTIME 10000

#define PSIZE 8

void fitssave(u_int *pix, char savefile[]);
void fitssave2(u_int *pix, char savefile[]);
void mouseFunc( int button, int status, int xx, int yy );
void keyboardFunc(int key, int status, int xx, int yy );
void displayFunc( void );
void MainLoop(void);
int exitstatus = 0;
int winid;
u_int *pix0, *pix1;
unsigned char *pixrgb;
long count;
long fitscount=1;
char fitshead[100];
char fitsname[100];

int sigmacounter=0;

int fd;
struct video_capability vd;
struct video_channel vc[10];
struct video_picture vp;
struct video_mbuf vm;
struct video_mmap vmm;

char *map, *p0, *p1;

float *x, *y;
int px, py;

char ccpusec[100];
float cpusec,cpusec0;

void displayFunc( void ){
  int i;
  int lx,ly,lx1,lx2,ly1,ly2;
  
  glRasterPos2i( 0, 0 );
  glDrawPixels( WIDTH/DISPSCALE, HEIGHT/DISPSCALE, GL_RGB, GL_UNSIGNED_BYTE, pixrgb );

  /* Plot Clicked Position */
  glColor3d(0.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2d(px-PSIZE/DISPSCALE, py-PSIZE/DISPSCALE);
  glVertex2d(px+PSIZE/DISPSCALE+1, py+PSIZE/DISPSCALE+1);
  glVertex2d(px-PSIZE/DISPSCALE, py+PSIZE/DISPSCALE);
  glVertex2d(px+PSIZE/DISPSCALE+1, py-PSIZE/DISPSCALE-1);
  glEnd();
    
  glFlush();
  glutSwapBuffers();
}

void mouseFunc( int button, int status, int xx, int yy ){
  if( button == GLUT_LEFT_BUTTON && status == GLUT_DOWN ){
    fprintf(stderr,"Pointer at (%d %d)\n",xx,yy);
    px=xx;py=HEIGHT/DISPSCALE-yy;
  }
  if( button == GLUT_RIGHT_BUTTON && status == GLUT_DOWN ){
    fprintf(stderr,"Cleared\n");
    px=-10;py=-10;
  }
  if( button == GLUT_MIDDLE_BUTTON && status == GLUT_DOWN ){
    fprintf(stderr,"Current COG at (%f %f)\n",x[0],y[0]);
    px=x[0]/DISPSCALE;py=HEIGHT/DISPSCALE-y[0]/DISPSCALE;
  }
}

void keyboardFunc(int key, int status, int xx, int yy ){
  switch (key) {
  case 'q':
  case 'Q':
    exitstatus = 1;
    break;
  case 's' :
    sprintf(fitsname,"%s.%03d.fits",fitshead,fitscount); 
    fitssave(pix0,fitsname); 
    fitscount++;
    break;
  case 'S' :
    sprintf(fitsname,"%s.%03d.fits",fitshead,fitscount); 
    fitssave2(pix0,fitsname); 
    fitscount++;
    break;
  case 'h':
  case '?':
    fprintf(stderr,"\n<Help>\n");
    fprintf(stderr,"q,Q: Quit this program.\n");
    fprintf(stderr,"s: Save current frame into FITS file.\n");
    fprintf(stderr,"S: Save current frame into 2 FITS files,.\n");
    fprintf(stderr,"   separated by odd/even lines.\n");
    fprintf(stderr,"H,h,?: Show this message.\n");
    break;
  default:
    break;
  }
}

int main(int argc, char **argv){
  int i,j,n;
  char tmp;
  int channel;

  char savefile[100];
  long sum;

  struct timeval cputime;
  struct timezone cpuzone; 

  time_t now;
  struct tm *now_utc;

  /* Clearpage */
  fprintf(stderr,"\033[2J");

  /* set cpusec0 */
  gettimeofday(&cputime, &cpuzone); 
  sprintf(ccpusec,"%d.%06d",cputime.tv_sec,cputime.tv_usec); 
  cpusec0=atof(ccpusec); 

  /*Get UTC and set it to be the header of FITS file*/
  now=time(NULL);
  now_utc=gmtime(&now);
  sprintf(fitshead,"%04d%02d%02d%02d%02d%02d",now_utc->tm_year+1900,now_utc->tm_mon+1,now_utc->tm_mday,now_utc->tm_hour,now_utc->tm_min,now_utc->tm_sec);


  pix0=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));
  pix1=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));
  pixrgb=(unsigned char *)malloc(WIDTH*HEIGHT*sizeof(char)*3/DISPSCALE/DISPSCALE);
  x=(float *) malloc(5*sizeof(float));
  y=(float *) malloc(5*sizeof(float));

  if(argc!=1){
    fprintf(stderr,"Usage : %s\n",argv[0]);
    exit(1);
  }

  /* Video4Linux ǥХΥץ */
  if((fd=open(VIDEODEVICE, O_RDWR))<0) {
    perror("open");
    exit(1);
  }
  
  /* Video4Linux ǥХͤμ */
  if(ioctl(fd, VIDIOCGCAP, &vd)<0) {
    perror("ioctl(VIDIOCGCAP)");
    exit(1);
  }
  /*  fprintf(stderr, "vd.name: \"%s\"\n", vd.name);
      fprintf(stderr, "vd.type=0x%08x", vd.type); 
  if(vd.type&VID_TYPE_CAPTURE) fprintf(stderr, " CAPTURE");
  if(vd.type&VID_TYPE_TUNER) fprintf(stderr, " TUNER");
  if(vd.type&VID_TYPE_TELETEXT) fprintf(stderr, " TELETEXT");
  if(vd.type&VID_TYPE_OVERLAY) fprintf(stderr, " OVERLAY");
  if(vd.type&VID_TYPE_CHROMAKEY) fprintf(stderr, " CHROMAKEY");
  if(vd.type&VID_TYPE_CLIPPING) fprintf(stderr, " CLIPPING");
  if(vd.type&VID_TYPE_FRAMERAM) fprintf(stderr, " FRAMERAM");
  if(vd.type&VID_TYPE_SCALES) fprintf(stderr, " SCALES");
  if(vd.type&VID_TYPE_MONOCHROME) fprintf(stderr, " MONOCHROME");
  if(vd.type&VID_TYPE_SUBCAPTURE) fprintf(stderr, " SUBCAPTURE");
  fprintf(stderr, "\n");
  fprintf(stderr, "vd.channels=%d\n", vd.channels);
  fprintf(stderr, "vd.audios=%d\n", vd.audios);
  fprintf(stderr, "vd.maxwidth=%d\n", vd.maxwidth);
  fprintf(stderr, "vd.maxheight=%d\n", vd.maxheight);
  fprintf(stderr, "vd.minwidth=%d\n", vd.minwidth);
  fprintf(stderr, "vd.minheight=%d\n", vd.minheight);
  fprintf(stderr, "\n");*/
  
  /* ƥͥͼ */
  for(n=0; n<vd.channels; n++) {
    vc[n].channel=n;
    if(ioctl(fd, VIDIOCGCHAN, &vc[n])<0) {
      perror("ioctl(VIDIOCGCHAN)");
      exit(1);
    }
    /*    fprintf(stderr, "vc[%d].channel=%d\n", n, vc[n].channel);
    fprintf(stderr, "vc[%d].name=\"%s\"\n", n, vc[n].name);
    fprintf(stderr, "vc[%d].tuners=%d\n", n, vc[n].tuners);
    fprintf(stderr, "vc[%d].flags=0x%08x", n, vc[n].flags);
    if(vc[n].flags&VIDEO_VC_TUNER) fprintf(stderr, " TUNER");
    if(vc[n].flags&VIDEO_VC_AUDIO) fprintf(stderr, " AUDIO");
    fprintf(stderr, "\n");
    fprintf(stderr, "vc[%d].type=0x%08x", n, vc[n].type);
    if(vc[n].type&VIDEO_TYPE_TV) fprintf(stderr, " TV");
    if(vc[n].type&VIDEO_TYPE_CAMERA) fprintf(stderr, " CAMERA");
    fprintf(stderr, "\n");
    fprintf(stderr, "vc[%d].norm=%d\n", n, vc[n].norm);
    fprintf(stderr, "\n");*/
  }
  
  /* "S-Video"پ channel=2 */
  /* "Composite1" channel=1 */
  channel=2;
  vc[channel].norm=1;/* 0:PAL 1:NTSC 2:SECAM */
  if(ioctl(fd, VIDIOCSCHAN, &vc[channel])<0) {
    perror("ioctl(VIDIOCSCHAN)");
    exit(1);
  }
  
  /* ĴȤ( 0-65535) */
  vp.brightness=32767;
  vp.hue=32767;
  vp.colour=32767;
  vp.contrast=33000;
  vp.whiteness=32767;
  
  /* color depth */
  vp.depth=8;
  /* ѥåȷ */
  vp.palette=VIDEO_PALETTE_GREY;
  if(ioctl(fd, VIDIOCSPICT, &vp)) {
    perror("ioctl(VIDIOCSPICT)");
    exit(1);
  }
  
  /* mmap μ */
  if(ioctl(fd, VIDIOCGMBUF, &vm)<0) {
    perror("ioctl(VIDIOCGMBUF)");
    exit(1);
  }
  /*  fprintf(stderr, "vm.size=0x%08x\n", vm.size);
      fprintf(stderr, "vm.frames=0x%08x\n", vm.frames); 
  for(n=0; n<vm.frames; n++)
    fprintf(stderr, "vm.offsets[%d]=0x%08x\n", n, vm.offsets[n]);
    fprintf(stderr, "\n");*/
  
  /* mmap */
  if((map=mmap(0, vm.size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0))==(char *)-1) {
    perror("mmap");
    exit(1);
  }
  
  /* specify format for vmm */
  vmm.width=WIDTH;
  vmm.height=HEIGHT;
  vmm.format=VIDEO_PALETTE_GREY;
  
  /* frame #0 ˥ץ㳫ϻؼ */
  vmm.frame=0;
  if(ioctl(fd, VIDIOCMCAPTURE, &vmm)<0) {
    perror("ioctl(VIDIOCMCAPTURE)");
    exit(1);
  }

  /* OpenGL  */
  glutInit( &argc, argv );
  glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE );
  glutInitWindowPosition( 0, 0 );

  glutInitWindowSize(WIDTH/DISPSCALE, HEIGHT/DISPSCALE); 
  winid=glutCreateWindow("monitor_window");

  glOrtho( 0.0, WIDTH/DISPSCALE - 1.0, 0.0, HEIGHT/DISPSCALE - 1.0, -1.0, 1.0 );
  glPixelZoom( 1.0, 1.0 );

  glClearColor( 0.0, 0.0, 0.0, 0.0 );
  glClear( GL_COLOR_BUFFER_BIT );
  glutSwapBuffers();
  glFlush();
  
  glutDisplayFunc( displayFunc );
  glutMouseFunc( mouseFunc );
  glutKeyboardFunc( keyboardFunc );
  glutIdleFunc( MainLoop );

  glutMainLoop();
  return 0;
}

void MainLoop(void){
  int i,j,n;
  char tmp;
  struct timeval cputime;
  int calcx0,calcy0,calcwidth,calcheight;

  calcx0=1; calcy0=1;
  calcwidth=WIDTH; calcheight=HEIGHT;

  /* frame #1 ˥ץ㳫ϻؼ */
  vmm.frame=1;
  if(ioctl(fd, VIDIOCMCAPTURE, &vmm)<0) {
    perror("ioctl(VIDIOCMCAPTURE)");
    exit(1);
  }
  usleep(WAITTIME);
  /* frame #0 ץ㽪λԤ */
  n=0;
  if(ioctl(fd, VIDIOCSYNC, &n)<0) {
    perror("ioctl(VIDIOCSYNC)");
    exit(1);
  }
  for(i=0, p0=map+vm.offsets[n]; i<NPIX; i++, p0+=1) {
    if(*(p0)<0){
      pix0[i]=*(p0)+256;
    }
    else pix0[i]=*(p0);
  }

  /* Display Frame */
  for(i=0;i<HEIGHT/DISPSCALE;i++){
    for(j=0;j<WIDTH/DISPSCALE;j++){
      tmp=*(map+vm.offsets[n]+DISPSCALE*j+DISPSCALE*i*WIDTH);
      pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3]=tmp;
      pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+1]=tmp;
      pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+2]=tmp;
    }
  }
  displayFunc(); 

  /* frame #0 ˥ץ㳫ϻؼ */
  vmm.frame=0;
  if(ioctl(fd, VIDIOCMCAPTURE, &vmm)<0) {
    perror("ioctl(VIDIOCMCAPTURE)");
    exit(1);
  }
  usleep(WAITTIME);
  /* frame #1 ץ㽪λԤ */
  n=1;
  if(ioctl(fd, VIDIOCSYNC, &n)<0) {
    perror("ioctl(VIDIOCSYNC)");
    exit(1);
  }
  for(i=0, p1=map+vm.offsets[n]; i<NPIX; i++, p1+=1) {
    if(*(p1)<0) pix1[i]=*(p1)+256;
    else pix1[i]=*(p1);
  }
  
  /* Display Frame */
  for(i=0;i<HEIGHT/DISPSCALE;i++){
    for(j=0;j<WIDTH/DISPSCALE;j++){
      tmp=*(map+vm.offsets[n]+DISPSCALE*j+DISPSCALE*i*WIDTH);
      pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3]=tmp;
      pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+1]=tmp;
      pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+2]=tmp;
    }
  }
  displayFunc(); 

  count++;

  if(exitstatus){
    fprintf(stderr,"\nExiting...\n");
    close(fd);
    exit(0);
  }
}



void fitssave(u_int *pix, char savefile[]){
  fitsfile *fptr;
  int fiostatus, nfound, anynull;
  long fpixel, npixels;
  int bitpix=8;
  long naxis=2;

  long naxes[2]={WIDTH, HEIGHT};

  /*FITS ؤν񤭽Ф */
  fprintf(stderr,"%s\n",savefile);
  unlink(savefile);
  fiostatus=0;
  if (fits_create_file(&fptr, savefile, &fiostatus)){ 
    fits_report_error(stderr,fiostatus); 
    exit(1);
  } 
  if (fits_create_img(fptr, bitpix, naxis, naxes, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fpixel=1;
  if (fits_write_img(fptr, TUINT, fpixel, NPIX, pix, &fiostatus)) 
    fits_report_error(stderr,fiostatus);
  if (fits_close_file(fptr, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
}

void fitssave2(u_int *pix, char savefile[]){
  fitsfile *fptr;
  int fiostatus, nfound, anynull;
  long fpixel, npixels;
  int bitpix=8;
  long naxis=2;
  u_int *lpix1,*lpix2;
  int i,j;
  char savefile1[100],savefile2[100];

  long naxes[2]={WIDTH, HEIGHT/2};

  lpix1=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));
  lpix2=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));

  for(j=0;j<HEIGHT/2;j++)
    for(i=0;i<WIDTH;i++)
      lpix1[i+j*WIDTH]=pix[i+j*2*WIDTH];
  for(j=0;j<HEIGHT/2;j++)
    for(i=0;i<WIDTH;i++)
      lpix2[i+j*WIDTH]=pix[i+(j*2+1)*WIDTH];
  
  sprintf(savefile1,"a.%s",savefile);
  sprintf(savefile2,"b.%s",savefile);

  /*FITS ؤν񤭽Ф */
  unlink(savefile1);
  fiostatus=0;
  if (fits_create_file(&fptr, savefile1, &fiostatus)){ 
    fits_report_error(stderr,fiostatus); 
    exit(1);
  } 
  if (fits_create_img(fptr, bitpix, naxis, naxes, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fpixel=1;
  if (fits_write_img(fptr, TUINT, fpixel, NPIX, lpix1, &fiostatus)) 
    fits_report_error(stderr,fiostatus);
  if (fits_close_file(fptr, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fprintf(stderr,"Halfsize image saved as %s\n",savefile1);

  unlink(savefile2);
  fiostatus=0;
  if (fits_create_file(&fptr, savefile2, &fiostatus)){ 
    fits_report_error(stderr,fiostatus); 
    exit(1);
  } 
  if (fits_create_img(fptr, bitpix, naxis, naxes, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fpixel=1;
  if (fits_write_img(fptr, TUINT, fpixel, NPIX, lpix2, &fiostatus)) 
    fits_report_error(stderr,fiostatus);
  if (fits_close_file(fptr, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fprintf(stderr,"Halfsize image saved as %s\n",savefile2);

  free(lpix1);
  free(lpix2);
}

