#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>

#include <linux/types.h>
#include <linux/videodev.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "fitsio.h"

#include <sys/time.h>
#include <time.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/* For RS232-C */
#include <termio.h>

#define WIDTH 640
#define HEIGHT 480
#define WIDTH2 640
#define HEIGHT2 480
/*#define DISPSCALE 2 */
#define DISPSCALE 1
#define NPIX 307200
#define WAITTIME 10000

#define CALCWIDTH 200
#define CALCHEIGHT 200
#define MASKWIDTH 20
#define PSIZE 8

#define MINSEP 5

#define SCLX 0.667 /* arcsec/pix */ 
#define SCLY 0.629 /* arcsec/pix */ 

#define MAXSIGMA 60

#define LAMBDA 0.5e-6
/*#define DD 0.08 */
#define DD 0.074
#define D 0.205

#define GNUPLOT_PATH "/usr/bin/gnuplot"
#define TMPDATA "/tmp/seeingdata.tmp"

#define MAXERROR 30

/* For RS232-C */
#define FIFO_RS232C "/dev/ttyS0"
#define PERMS 0666

void glDrawCircle(int x, int y, float radius);
void calccogtot(int *pix);
int calccog1(int *pix, float *xa, float *ya);
int calccog2(int *pix, int width, int height, float *xa, float *ya);
int calccog2slowinterlace(int *pix, int width, int height, float *xa, float *ya, int line);
int calccog3slowinterlace(int *pix, int width, int height, float *xa, float *ya, int line);
int calccog0slowinterlace(int *pix, int width, int height, float *xa, float *ya, int line);
float calcbackground(int *pix, int x0, int y0, int width, int height);
void fitssave(u_int *pix, char savefile[]);
void fitssave2(u_int *pix, char savefile[]);
void mouseFunc( int button, int status, int xx, int yy );
void keyboardFunc(int key, int status, int xx, int yy );
void displayFunc( void );
void MainLoop(void);

/* Functions to controll MEADE telescope */
int openSio(void);
int closeSio(void);
int sendCom(char *tx);
int setSlewRate(unsigned int sp);
int AutoGuide(void);
int setObj(int num);
double getAlt(void);
double getAz(void);

int comstatus = 0;
int aslewstatus = 0; /* altitude */
int zslewstatus = 0; /* azimuth */
int cflag,aflag,gnflag,geflag; /* c:comm, a:a/g, g:guide */

int tgx = WIDTH/2;  /* AG target coordinates */
int tgy = HEIGHT/2; /* AG target coordinates */
int nz = 30; /* Null zone for auto guide */
int agc = 0; /* AG counter */
double az = 0.0; /* Azimuth */

static int siofd; /* sio fd */
unsigned int slewrate = 1; /* set to GUIDE */
/* end */

int exitstatus = 0;
int flag,pflag,fflag;
int winid;
u_int *pix0, *pix1;
unsigned char *pixrgb;
long count;
long fitscount=1;
int window = 0; /* 0=off:1=on*/
char fitshead[100];
char fitsname[100];

int sigmacounter=0;
float xfsig1[MAXSIGMA],xfsig2[MAXSIGMA],xfsig3[MAXSIGMA],xfsig4[MAXSIGMA];
float yfsig1[MAXSIGMA],yfsig2[MAXSIGMA],yfsig3[MAXSIGMA],yfsig4[MAXSIGMA];

float inter0xfsig1[MAXSIGMA],inter0xfsig2[MAXSIGMA],inter0xfsig3[MAXSIGMA],inter0xfsig4[MAXSIGMA];
float inter0yfsig1[MAXSIGMA],inter0yfsig2[MAXSIGMA],inter0yfsig3[MAXSIGMA],inter0yfsig4[MAXSIGMA];

float inter1xfsig1[MAXSIGMA],inter1xfsig2[MAXSIGMA],inter1xfsig3[MAXSIGMA],inter1xfsig4[MAXSIGMA];
float inter1yfsig1[MAXSIGMA],inter1yfsig2[MAXSIGMA],inter1yfsig3[MAXSIGMA],inter1yfsig4[MAXSIGMA];

int fd;
struct video_capability vd;
struct video_channel vc[10];
struct video_picture vp;
struct video_mbuf vm;
struct video_mmap vmm;

char *map, *p0, *p1;

float *x, *y;
float *inter0x, *inter0y,*inter1x, *inter1y;
int px, py;

int nerror;

FILE *gp;
int gnuplotstatus=1; /* 0:on 1:off 2:failed*/
char tmpseeingdata[100];

char ccpusec[100];
float cpusec,cpusec0;

void displayFunc( void ){
  int i;
  int lx,ly,lx1,lx2,ly1,ly2;
  
  glRasterPos2i( 0, 0 );
  if(flag!=2 || window!=0)
    glDrawPixels( WIDTH/DISPSCALE, HEIGHT/DISPSCALE, GL_RGB, GL_UNSIGNED_BYTE, pixrgb );

  /* Plot CG */
  for(i=0;i<1;i++){
    lx=x[i]/DISPSCALE;
    ly=HEIGHT/DISPSCALE-y[i]/DISPSCALE;
    if(i==0) glColor3d(1.0, 0.0, 0.0);
    else glColor3d(1.0, 1.0, 1.0);
    glBegin(GL_LINES);
    glVertex2d(lx-PSIZE/DISPSCALE, ly-PSIZE/DISPSCALE);
    glVertex2d(lx+PSIZE/DISPSCALE+1, ly+PSIZE/DISPSCALE+1);
    glVertex2d(lx-PSIZE/DISPSCALE, ly+PSIZE/DISPSCALE);
    glVertex2d(lx+PSIZE/DISPSCALE+1, ly-PSIZE/DISPSCALE-1);
    glEnd();
  }

  for(i=1;i<5;i++){
    lx=x[i]/DISPSCALE;
    ly=HEIGHT/DISPSCALE-y[i]/DISPSCALE;
    glColor3d(1.0, 1.0, 1.0);
    glDrawCircle(lx, ly, 10/DISPSCALE);
  }

  if(sigmacounter>2){
    lx1=x[1]/DISPSCALE;
    ly1=HEIGHT/DISPSCALE-y[1]/DISPSCALE;
    lx2=x[2]/DISPSCALE;
    ly2=HEIGHT/DISPSCALE-y[2]/DISPSCALE;
    glColor3d(0.0, 0.0, 1.0);
    glBegin(GL_LINES);
    glVertex2d(lx1, ly1);
    glVertex2d(lx2, ly2);
    glEnd();

    /*    lx1=x[3]/DISPSCALE;
    ly1=HEIGHT/DISPSCALE-y[3]/DISPSCALE;
    lx2=x[4]/DISPSCALE;
    ly2=HEIGHT/DISPSCALE-y[4]/DISPSCALE;
    glColor3d(0.0, 1.0, 0.0);
    glBegin(GL_LINES);
    glVertex2d(lx1, ly1);
    glVertex2d(lx2, ly2);
    glEnd();
    */
  }

  /* Plot Clicked Position */
  glColor3d(0.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2d(px-PSIZE/DISPSCALE, py-PSIZE/DISPSCALE);
  glVertex2d(px+PSIZE/DISPSCALE+1, py+PSIZE/DISPSCALE+1);
  glVertex2d(px-PSIZE/DISPSCALE, py+PSIZE/DISPSCALE);
  glVertex2d(px+PSIZE/DISPSCALE+1, py-PSIZE/DISPSCALE-1);
  glEnd();

  /* If AG is working */
  if(aflag){
    /* Plot Target Position */
    if(gnflag>0 && geflag>0) glColor3d(0.0, 0.7, 0.0);
    else if(gnflag>0 || geflag>0) glColor3d(0.7, 0.7, 0.0);
    else glColor3d(0.7, 0.0, 0.0);
    glBegin(GL_LINES);
    glVertex2d(tgx-nz/DISPSCALE, HEIGHT/DISPSCALE-tgy+nz/DISPSCALE+1);
    glVertex2d(tgx-nz/DISPSCALE, HEIGHT/DISPSCALE-tgy-nz/DISPSCALE);
    glVertex2d(tgx+nz/DISPSCALE+1, HEIGHT/DISPSCALE-tgy-nz/DISPSCALE);
    glVertex2d(tgx+nz/DISPSCALE+1, HEIGHT/DISPSCALE-tgy+nz/DISPSCALE+1);
    glVertex2d(tgx-nz/DISPSCALE, HEIGHT/DISPSCALE-tgy-nz/DISPSCALE);
    glVertex2d(tgx+nz/DISPSCALE+1, HEIGHT/DISPSCALE-tgy-nz/DISPSCALE);
    glVertex2d(tgx-nz/DISPSCALE, HEIGHT/DISPSCALE-tgy+nz/DISPSCALE+1);
    glVertex2d(tgx+nz/DISPSCALE+1, HEIGHT/DISPSCALE-tgy+nz/DISPSCALE+1);
    glVertex2d(tgx-nz/DISPSCALE, HEIGHT/DISPSCALE-tgy+nz/DISPSCALE+1);
    glEnd();

    glLineWidth(1.0);
    glRasterPos2i(tgx+nz/DISPSCALE+2,HEIGHT/DISPSCALE-tgy+nz/DISPSCALE+2);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,'A');
  }

  /* If RS232-C link with telescope is active */
  if(cflag){
    /* Comm Status */
    if(aslewstatus || zslewstatus) glColor3d(1.0, 0.0, 0.0);
    else glColor3d(0.0, 1.0, 0.0);
    glLineWidth(1.0);
    glRasterPos2i(610/DISPSCALE,10/DISPSCALE);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,'C');
    glRasterPos2i(625/DISPSCALE,10/DISPSCALE);
    switch(slewrate){
    case 1:
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,'G');
      break;
    case 2:
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,'C');
      break;
    case 3:
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,'F');
      break;
    case 4:
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,'S');
      break;
    }
  }
   
    
  glFlush();
  glutSwapBuffers();
}

void mouseFunc( int button, int status, int xx, int yy ){
  if( button == GLUT_LEFT_BUTTON && status == GLUT_DOWN ){
    fprintf(stderr,"Pointer at (%d %d)\n",xx,yy);
    px=xx;py=HEIGHT/DISPSCALE-yy;
  }
  if( button == GLUT_RIGHT_BUTTON && status == GLUT_DOWN ){
    fprintf(stderr,"Cleared\n");
    px=-10;py=-10;
  }
  if( button == GLUT_MIDDLE_BUTTON && status == GLUT_DOWN ){
    fprintf(stderr,"Current COG at (%f %f)\n",x[0],y[0]);
    px=x[0]/DISPSCALE;py=HEIGHT/DISPSCALE-y[0]/DISPSCALE;
  }
}

void keyboardFunc(int key, int status, int xx, int yy ){
  switch (key) {
  case 'q':
  case 'Q':
    exitstatus = 1;
  case 'f':
    if(fflag==1){
      fflag=0;
      flag=0;
      fprintf(stderr,"\nUse Fast mode\n");
    }
    else if(fflag==0){
      fflag=1;
      flag=0;
      fprintf(stderr,"\nUse Slow mode\n");
    }
    break;
  case 'w':
    if(window==1) window=0;
    else if(window==0) window=1;
    break;
  case 'g':
    if(gnuplotstatus==1){
      gnuplotstatus=0;
      fprintf(stderr,"\nGraph mode is ON\n");
    }
    else if(gnuplotstatus==0){
      gnuplotstatus=1;
      fprintf(stderr,"\nGraph mode is OFF\n");
    }
    break;
  case 's' :
    sprintf(fitsname,"%s.%03d.fits",fitshead,fitscount); 
    fitssave(pix0,fitsname); 
    fitscount++;
    break;
  case 'S' :
    sprintf(fitsname,"%s.%03d.fits",fitshead,fitscount); 
    fitssave2(pix0,fitsname); 
    fitscount++;
    break;
  case 'r' :
    fprintf(stderr,"\nReset...\n");
    flag=0;
    break;
  /*Start Comminucation with telescope */
  case 'c' :
    if(cflag==1){
      cflag=0;
    }
    else if(cflag==0){
      cflag=1;
    }
    break;
  case 'h':
  case '?':
    fprintf(stderr,"\n<Help>\n");
    fprintf(stderr,"H,h,?: Show this message.\n");
    fprintf(stderr,"q,Q: Quit this program.\n");
    fprintf(stderr," [Data Acquisition]\n");
    fprintf(stderr,"  g: Show/hide graph plot of the seeing.\n");
    fprintf(stderr,"  w: Show/hide the current frame in the GLUT window.\n");
    fprintf(stderr,"   If you show the frame, sampling rate will become slow.\n");
    fprintf(stderr,"  f: Switch Fast/Slow mode.\n");
    fprintf(stderr,"  r: Reset all seeing/COG calculation.\n");
    fprintf(stderr,"  s: Save current frame into FITS file.\n");
    fprintf(stderr,"  S: Save current frame into 2 FITS files,.\n");
    fprintf(stderr,"     separated by odd/even lines.\n");
    fprintf(stderr," [Telescope operation]\n");
    fprintf(stderr,"  c: Open/close serial I/O device.\n");
    fprintf(stderr,"  Page Up: Increase slew rate. Enlarge null zone on auto guide.\n");
    fprintf(stderr,"  Page Dn: Decrease slew rate. Reduce null zone on auto guide.\n");
    fprintf(stderr,"  Up: Move telescope North, Halt north/southward slews.\n");
    fprintf(stderr,"  Down: Move telescope South, Halt north/southward slews.\n");
    fprintf(stderr,"  Left: Move telescope West, Halt east/westward slews.\n");
    fprintf(stderr,"  Right: Move telescope East, Halt east/westward slews.\n");
    fprintf(stderr,"  F1-F12: Set the object.\n");
    fprintf(stderr,"  Home: Start auto guide.\n");
    fprintf(stderr,"  Insert: Slew telescope to current object coordinates.\n");
    fprintf(stderr,"  End: Cancel all slew, auto guide.\n");
    break;
  default:
    break;
  }
}

/* Special Keys */
void specialFunc(unsigned char key, int status, int xx, int yy ){
  int i,j,n;
  char tmp;

  switch (key) {
  case GLUT_KEY_DOWN:
    if(aslewstatus){
      sendCom(":Qs#");sendCom(":Qn#");
      aslewstatus=0;
    }else{
      sendCom(":Ms#");
      aslewstatus=1;
    }
    break;
  case GLUT_KEY_UP:
    if(aslewstatus){
      sendCom(":Qs#");sendCom(":Qn#");
      aslewstatus=0;
    }else{
      sendCom(":Mn#");
      aslewstatus=1;
    }
    break;
  case GLUT_KEY_LEFT:
    if(zslewstatus){
      sendCom(":Qe#");sendCom(":Qw#");
      zslewstatus=0;
    }else{
      sendCom(":Mw#");
      zslewstatus=1;
    }
    break;
  case GLUT_KEY_RIGHT:
    if(zslewstatus){
      sendCom(":Qe#");sendCom(":Qw#");
      zslewstatus=0;
    }else{
      sendCom(":Me#");
      zslewstatus=1;
    }
    break;
  case GLUT_KEY_PAGE_UP:
    if(aflag) nz++;
    else setSlewRate(1);
    break;
  case GLUT_KEY_PAGE_DOWN:
    if(aflag) nz--;
    else setSlewRate(0);
    break;
  case GLUT_KEY_HOME:
    fprintf(stderr,"\nAG started\n");
    aflag=1;
    break;
  case GLUT_KEY_END:
    fprintf(stderr,"\nCommand cancelled\n");
    aflag=0; gnflag=1; geflag=1; aslewstatus=0; zslewstatus=0;
    sendCom(":Q#");
    sendCom(":Qe#");sendCom(":Qw#");
    sendCom(":Qs#");sendCom(":Qn#");
    break;
  case GLUT_KEY_F1:
    setObj(17);
    break;
  case GLUT_KEY_F2:
    setObj(33);
    break;
  case GLUT_KEY_F3:
    setObj(42);
    break;
  case GLUT_KEY_F4:
    setObj(56);
    break;
  case GLUT_KEY_F5:
    setObj(81);
    break;
  case GLUT_KEY_F6:
    setObj(100);
    break;
  case GLUT_KEY_F7:
    setObj(114);
    break;
  case GLUT_KEY_F8:
    setObj(138);
    break;
  case GLUT_KEY_F9:
    setObj(147);
    break;
  case GLUT_KEY_F10:
    setObj(214);
    break;
  case GLUT_KEY_F11:
    setObj(226);
    break;
  case GLUT_KEY_F12:
    setObj(232);
    break;
  case GLUT_KEY_INSERT:
    sendCom(":MS#");
    aslewstatus=1;zslewstatus=1;
    break;
  }
}

int main(int argc, char **argv){
  int i,j,n;
  char tmp;
  int channel;

  char savefile[100];
  long sum;

  struct timeval cputime;
  struct timezone cpuzone; 

  time_t now;
  struct tm *now_utc;

  if(argc != 2 && argc != 1){
    fprintf(stderr,"Usage: %s [Channel]\n",argv[0]);
    fprintf(stderr," Default channel is 1\n");
    exit(1);
  }

  if( argc == 1 ) channel=1;
  if( argc == 2 ) channel=atoi(argv[1]);

  /* Clearpage */
  fprintf(stderr,"\033[2J");

  /* set cpusec0 */
  gettimeofday(&cputime, &cpuzone); 
  sprintf(ccpusec,"%d.%06d",cputime.tv_sec,cputime.tv_usec); 
  cpusec0=atof(ccpusec); 

  /* Open GNUPLOT */
  gp = popen(GNUPLOT_PATH, "w");
  if(gp == NULL){
    fprintf(stderr, "Warning :  Can't find %s.", GNUPLOT_PATH);
    gnuplotstatus=2;
  }
  else gnuplotstatus=1;

  /* Set temporary filename*/
  sprintf(tmpseeingdata,"%s",TMPDATA);
  unlink(tmpseeingdata);

  /*Get UTC and set it to be the header of FITS file*/
  now=time(NULL);
  now_utc=gmtime(&now);
  sprintf(fitshead,"%04d%02d%02d%02d%02d%02d",now_utc->tm_year+1900,now_utc->tm_mon+1,now_utc->tm_mday,now_utc->tm_hour,now_utc->tm_min,now_utc->tm_sec);


  pix0=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));
  pix1=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));
  pixrgb=(unsigned char *)malloc(WIDTH*HEIGHT*sizeof(char)*3/DISPSCALE/DISPSCALE);
  x=(float *) malloc(5*sizeof(float));
  y=(float *) malloc(5*sizeof(float));
  inter0x=(float *) malloc(5*sizeof(float));
  inter0y=(float *) malloc(5*sizeof(float));
  inter1x=(float *) malloc(5*sizeof(float));
  inter1y=(float *) malloc(5*sizeof(float));

  /* Video4Linux ǥХΥץ */
  if((fd=open("/dev/video0", O_RDWR))<0) {
    perror("open");
    exit(1);
  }
  
  /* Video4Linux ǥХͤμ */
  if(ioctl(fd, VIDIOCGCAP, &vd)<0) {
    perror("ioctl(VIDIOCGCAP)");
    exit(1);
  }
  /*fprintf(stderr, "vd.name: \"%s\"\n", vd.name);
      fprintf(stderr, "vd.type=0x%08x", vd.type); 
  if(vd.type&VID_TYPE_CAPTURE) fprintf(stderr, " CAPTURE");
  if(vd.type&VID_TYPE_TUNER) fprintf(stderr, " TUNER");
  if(vd.type&VID_TYPE_TELETEXT) fprintf(stderr, " TELETEXT");
  if(vd.type&VID_TYPE_OVERLAY) fprintf(stderr, " OVERLAY");
  if(vd.type&VID_TYPE_CHROMAKEY) fprintf(stderr, " CHROMAKEY");
  if(vd.type&VID_TYPE_CLIPPING) fprintf(stderr, " CLIPPING");
  if(vd.type&VID_TYPE_FRAMERAM) fprintf(stderr, " FRAMERAM");
  if(vd.type&VID_TYPE_SCALES) fprintf(stderr, " SCALES");
  if(vd.type&VID_TYPE_MONOCHROME) fprintf(stderr, " MONOCHROME");
  if(vd.type&VID_TYPE_SUBCAPTURE) fprintf(stderr, " SUBCAPTURE");
  fprintf(stderr, "\n");
  fprintf(stderr, "vd.channels=%d\n", vd.channels);
  fprintf(stderr, "vd.audios=%d\n", vd.audios);
  fprintf(stderr, "vd.maxwidth=%d\n", vd.maxwidth);
  fprintf(stderr, "vd.maxheight=%d\n", vd.maxheight);
  fprintf(stderr, "vd.minwidth=%d\n", vd.minwidth);
  fprintf(stderr, "vd.minheight=%d\n", vd.minheight);
  fprintf(stderr, "\n");*/
  
  /* ƥͥͼ */
  for(n=0; n<vd.channels; n++) {
    vc[n].channel=n;
    if(ioctl(fd, VIDIOCGCHAN, &vc[n])<0) {
      perror("ioctl(VIDIOCGCHAN)");
      exit(1);
    }
    /*    fprintf(stderr, "vc[%d].channel=%d\n", n, vc[n].channel);
    fprintf(stderr, "vc[%d].name=\"%s\"\n", n, vc[n].name);
    fprintf(stderr, "vc[%d].tuners=%d\n", n, vc[n].tuners);
    fprintf(stderr, "vc[%d].flags=0x%08x", n, vc[n].flags);
    if(vc[n].flags&VIDEO_VC_TUNER) fprintf(stderr, " TUNER");
    if(vc[n].flags&VIDEO_VC_AUDIO) fprintf(stderr, " AUDIO");
    fprintf(stderr, "\n");
    fprintf(stderr, "vc[%d].type=0x%08x", n, vc[n].type);
    if(vc[n].type&VIDEO_TYPE_TV) fprintf(stderr, " TV");
    if(vc[n].type&VIDEO_TYPE_CAMERA) fprintf(stderr, " CAMERA");
    fprintf(stderr, "\n");
    fprintf(stderr, "vc[%d].norm=%d\n", n, vc[n].norm);
    fprintf(stderr, "\n"); */
  }
  
  /* "S-Video"پ channel=2 */
  /* "Composite1" channel=1 <- ǻ */
  vc[channel].norm=1;/* 0:PAL 1:NTSC 2:SECAM */
  if(ioctl(fd, VIDIOCSCHAN, &vc[channel])<0) {
    perror("ioctl(VIDIOCSCHAN)");
    exit(1);
  }
  
  fprintf(stderr,"Channel=%d (%s)\n",channel,vc[channel].name);

  /* ĴȤ( 0-65535) */
  vp.brightness=32767;
  vp.hue=32767;
  vp.colour=32767;
  vp.contrast=33000;
  vp.whiteness=32767;
  
  /* color depth */
  vp.depth=8;
  /* ѥåȷ */
  vp.palette=VIDEO_PALETTE_GREY;
  if(ioctl(fd, VIDIOCSPICT, &vp)) {
    perror("ioctl(VIDIOCSPICT)");
    exit(1);
  }
  
  /* mmap μ */
  if(ioctl(fd, VIDIOCGMBUF, &vm)<0) {
    perror("ioctl(VIDIOCGMBUF)");
    exit(1);
  }
  /*  fprintf(stderr, "vm.size=0x%08x\n", vm.size);
      fprintf(stderr, "vm.frames=0x%08x\n", vm.frames); 
  for(n=0; n<vm.frames; n++)
    fprintf(stderr, "vm.offsets[%d]=0x%08x\n", n, vm.offsets[n]);
    fprintf(stderr, "\n");*/
  
  /* mmap */
  if((map=mmap(0, vm.size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0))==(char *)-1) {
    perror("mmap");
    exit(1);
  }
  
  /* specify format for vmm */
  vmm.width=WIDTH;
  vmm.height=HEIGHT;
  vmm.format=VIDEO_PALETTE_GREY;
  
  /* frame #0 ˥ץ㳫ϻؼ */
  vmm.frame=0;
  if(ioctl(fd, VIDIOCMCAPTURE, &vmm)<0) {
    perror("ioctl(VIDIOCMCAPTURE)");
    exit(1);
  }

  flag=0;
  /* OpenGL  */
  glutInit( &argc, argv );
  glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE );
  glutInitWindowPosition( 0, 0 );

  glutInitWindowSize(WIDTH/DISPSCALE, HEIGHT/DISPSCALE); 
  /*  glutInitWindowSize(WIDTH, HEIGHT); */
  winid=glutCreateWindow("monitor_window");

  glOrtho( 0.0, WIDTH/DISPSCALE - 1.0, 0.0, HEIGHT/DISPSCALE - 1.0, -1.0, 1.0 );
  glPixelZoom( 1.0, 1.0 );

  glClearColor( 0.0, 0.0, 0.0, 0.0 );
  glClear( GL_COLOR_BUFFER_BIT );
  glutSwapBuffers();
  glFlush();
  
  glutDisplayFunc( displayFunc );
  glutMouseFunc( mouseFunc );
  glutKeyboardFunc( keyboardFunc );
  glutSpecialFunc( specialFunc );
  glutIdleFunc( MainLoop );

  glutMainLoop();
  return 0;
}

void MainLoop(void){
  int i,j,n;
  char tmp;
  struct timeval cputime;
  int calcx0,calcy0,calcwidth,calcheight;

  calcx0=1; calcy0=1;
  calcwidth=WIDTH; calcheight=HEIGHT;

  /* Open/Close RS232-C port */
  if(cflag){
    if(!comstatus) openSio();
  }
  else{
    if(comstatus) closeSio();
  }

  /* frame #1 ˥ץ㳫ϻؼ */
  vmm.frame=1;
  if(ioctl(fd, VIDIOCMCAPTURE, &vmm)<0) {
    perror("ioctl(VIDIOCMCAPTURE)");
    exit(1);
  }
  usleep(WAITTIME);
  /* frame #0 ץ㽪λԤ */
  n=0;
  if(ioctl(fd, VIDIOCSYNC, &n)<0) {
    perror("ioctl(VIDIOCSYNC)");
    exit(1);
  }
  /*  ftime(&cputime); */
  /*  if(flag==2) fprintf(stdout,"%d.%03d\t",cputime.time,cputime.millitm); */
  for(i=0, p0=map+vm.offsets[n]; i<NPIX; i++, p0+=1) {
    if(*(p0)<0){
      pix0[i]=*(p0)+256;
    }
    else pix0[i]=*(p0);
  }

  /* Calculate COG and change mode */
  calccogtot(pix0);
  
  /* Display Frame */
  if((count%1==0 && window!=0) || flag!=2){
    for(i=0;i<HEIGHT/DISPSCALE;i++){
      for(j=0;j<WIDTH/DISPSCALE;j++){
	tmp=*(map+vm.offsets[n]+DISPSCALE*j+DISPSCALE*i*WIDTH);
	pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3]=tmp;
	pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+1]=tmp;
	pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+2]=tmp;
      }
    }
  }
  displayFunc(); 

  /* frame #0 ˥ץ㳫ϻؼ */
  vmm.frame=0;
  if(ioctl(fd, VIDIOCMCAPTURE, &vmm)<0) {
    perror("ioctl(VIDIOCMCAPTURE)");
    exit(1);
  }
  usleep(WAITTIME);
  /* frame #1 ץ㽪λԤ */
  n=1;
  if(ioctl(fd, VIDIOCSYNC, &n)<0) {
    perror("ioctl(VIDIOCSYNC)");
    exit(1);
  }
  /*  ftime(&cputime);  */
  /*  if(flag==2) fprintf(stdout,"%d.%03d\t",cputime.time,cputime.millitm); */
  for(i=0, p1=map+vm.offsets[n]; i<NPIX; i++, p1+=1) {
    if(*(p1)<0) pix1[i]=*(p1)+256;
    else pix1[i]=*(p1);
  }

  /* Calculate COG and change mode */
  calccogtot(pix1);
  
  /* Display Frame */
  if((count%1==0 && window!=0) || flag!=2){
    for(i=0;i<HEIGHT/DISPSCALE;i++){
      for(j=0;j<WIDTH/DISPSCALE;j++){
	tmp=*(map+vm.offsets[n]+DISPSCALE*j+DISPSCALE*i*WIDTH);
	pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3]=tmp;
	pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+1]=tmp;
	pixrgb[(j+(HEIGHT/DISPSCALE-1-i)*WIDTH/DISPSCALE)*3+2]=tmp;
      }
    }
  }
  displayFunc(); 

  /* Auto Guiding */
  if(aflag) AutoGuide();

  count++;

  if(exitstatus){
    fprintf(stderr,"\nExiting...\n");
    /*    fprintf(stderr,"Save final frame to\n");
	  fitssave(pix0,"test0.fits");
	  fitssave(pix1,"test1.fits");  */
    /* ppm ν񤭤 */
    /*    printf("P6 %d %d 255\n", WIDTH, HEIGHT); fflush(stdout);  */
    /*    write(1, pixrgb, WIDTH*HEIGHT*3);  */
    close(fd);
    pclose(gp);
    exit(0);
  }
}

int calccog1(int *pix, float *xa, float *ya){
  int i,j;
  float av;
  float sum;
  int lx, ly;
  int npix;
  int lpix;
  struct timeval cputime;
  struct timezone cpuzone;
  
  gettimeofday(&cputime, &cpuzone); 
  npix=WIDTH*HEIGHT;

  for(i=0;i<5;i++){
    xa[i]=0;ya[i]=0;
  }

  /* Calculate Background */
  av=0;
  sum=0;
  for(j=HEIGHT/8;j<HEIGHT/8*3;j++){
    for(i=WIDTH/8;i<WIDTH/8*3;i++){
      sum+=pix[i+j*WIDTH];
    }
  }
  av=sum;
  sum=0;
  for(j=HEIGHT/8*5;j<HEIGHT/8*7;j++){
    for(i=WIDTH/8;i<WIDTH/8*3;i++){
      sum+=pix[i+j*WIDTH];
    }
  }
  if(av>sum)av=sum;
  sum=0;
  for(j=HEIGHT/8*5;j<HEIGHT/8*7;j++){
    for(i=WIDTH/8*5;i<WIDTH/8*7;i++){
      sum+=pix[i+j*WIDTH];
    }
  }
  if(av>sum)av=sum;
  sum=0;
  for(j=HEIGHT/8;j<HEIGHT/8*3;j++){
    for(i=WIDTH/8*5;i<WIDTH/8*7;i++){
      sum+=pix[i+j*WIDTH];
    }
  }
  if(av>sum)av=sum;
  av/=(npix/16);
  
  /* Calculate COG */
  sum=0;
  for(j=0;j<HEIGHT;j++){
    for(i=0;i<WIDTH;i++){
      lpix=pix[i+j*WIDTH]-av;
      xa[0]+=(float) i*lpix;
      ya[0]+=(float) j*lpix;
      sum+=lpix;
    }
  }
  if(sum!=0){
    xa[0]/=sum;ya[0]/=sum;
  }
  else{
    xa[0]=0;ya[0]=0;
  }
  fprintf(stderr,"%d.%06d\t",cputime.tv_sec,cputime.tv_usec);
  fprintf(stderr,"MODE1 : av=%e\t cog=(%f, %f)\r",av,xa[0],ya[0]);
  if(xa[0]>CALCWIDTH/4 && xa[0]< WIDTH-CALCWIDTH/4 && ya[0]>CALCHEIGHT/4 && ya[0]< HEIGHT-CALCHEIGHT/4){
    return 0;
  }
  else
    return 1;
}

int calccog2(int *pix, int width, int height, float *xa, float *ya){
  int i,j;
  float av;
  float sum;
  float d;
  int lx, ly;
  int x0,y0,x1,x2,y1,y2;
  int xmax,ymax;
  int maxval;
  int npix;
  int lpix;
  float d1,d2,costheta,degree;
  struct timeval cputime;
  struct timezone cpuzone;

  gettimeofday(&cputime, &cpuzone); 
  npix=width*height;

  /*  fprintf(stderr,"\nxa ya %f %f\n",xa[0],ya[0]); */
  x0=xa[0]; y0=ya[0];
  for(i=0;i<5;i++){
    xa[i]=0;ya[i]=0;
  }

  /* Calculate Background */
  /*  av=calcbackground(pix,xa[0],ya[0],width, height); */
  av=calcbackground(pix,x0,y0,width, height);
  
  /* Calculate COG */
  x1=x0-0.5*width; if(x1<0) x1=0;
  x2=x0+0.5*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=y0-0.5*height; if(y1<0) y1=0;
  y2=y0+0.5*height; if(y2>HEIGHT-1) y2=HEIGHT-1;

  npix=0;
  sum=0;
  maxval=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+j*WIDTH]-av;
      xa[0]+=(float) i*lpix;
      ya[0]+=(float) j*lpix;
      sum+=lpix;
      npix++;
      /*
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j;
      }
      */
    }
  }
  if(sum!=0){
    xa[0]/=sum;ya[0]/=sum;
  }
  else{
    xa[0]=0;ya[0]=0;
  }

  if(xa[0]>x2 || xa[0]<x1){fprintf(stderr,"error cog2.1x xa[0]=%d x1=%d x2=%d\n",xa[0],x1,x2); return 1;}
  if(ya[0]>y2 || ya[0]<y1){fprintf(stderr,"error cog2.1y ya[0]=%d y1=%d y2=%d\n",ya[0],y1,y2); return 1;}

  /*
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog2.2x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog2.2y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  */

  /* Detect Peak 1 --- the Highest Peak in the 1-4th quadrant */
  x1=xa[0]-0.25*width; if(x1<0) x1=0;
  x2=xa[0]+0.25*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=ya[0]-0.5*height; if(y1<0) y1=0;
  y2=ya[0]-0.05*height; if(y2>HEIGHT-1) y2=HEIGHT-1;

  maxval=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+j*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j;
      }
    }
  }
  
  xa[1]=xmax;
  ya[1]=ymax;

  /* Detect Peak 2 -- crossing of xa[1] */
  x1=xmax-MASKWIDTH; if(x1<0) x1=0;
  x2=xmax+MASKWIDTH; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=ymax-MASKWIDTH; if(y1<0) y1=0;
  y2=ymax+MASKWIDTH; if(y2>HEIGHT-1) y2=HEIGHT-1;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      pix[i+j*WIDTH]=av;
    }
  }
  maxval=0;

  x1=2*xa[0]-xa[1]-MASKWIDTH; if(x1<0) x1=0;
  x2=2*xa[0]-xa[1]+MASKWIDTH; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=2*ya[0]-ya[1]-MASKWIDTH; if(y1<0) y1=0;
  y2=2*ya[0]-ya[1]+MASKWIDTH; if(y2>HEIGHT-1) y2=HEIGHT-1;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+j*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog2.3x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog2.3y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[2]=xmax;
  ya[2]=ymax;

  /* Detect Peak 3*/
  x1=xmax-MASKWIDTH; if(x1<0) x1=0;
  x2=xmax+MASKWIDTH; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=ymax-MASKWIDTH; if(y1<0) y1=0;
  y2=ymax+MASKWIDTH; if(y2>HEIGHT-1) y2=HEIGHT-1;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      pix[i+j*WIDTH]=av;
    }
  }

  x1=x[0]-0.5*width; if(x1<0) x1=0;
  x2=x[0]+0.5*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=y[0]-0.5*height; if(y1<0) y1=0;
  y2=y[0]+0.5*height; if(y2>HEIGHT-1) y2=HEIGHT-1;
  maxval=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+j*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog2.4x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog2.4y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[3]=xmax;
  ya[3]=ymax;
  
  for(i=1;i<=2;i++){
    d=sqrt((xa[3]-xa[i])*(xa[3]-xa[i])+(ya[3]-ya[i])*(ya[3]-ya[i]));
    if(d<MINSEP){fprintf(stderr,"error cog2.4.%d; separation %f\n",i,d); return 1;}
  }

  /* Detect Peak 4 */
  x1=xmax-MASKWIDTH; if(x1<0) x1=0;
  x2=xmax+MASKWIDTH; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=ymax-MASKWIDTH; if(y1<0) y1=0;
  y2=ymax+MASKWIDTH; if(y2>HEIGHT-1) y2=HEIGHT-1;
  for(j=ymax-MASKWIDTH;j<ymax+MASKWIDTH;j++){
    for(i=xmax-MASKWIDTH;i<xmax+MASKWIDTH;i++){
      pix[i+j*WIDTH]=av;
    }
  }

  x1=x[0]-0.5*width; if(x1<0) x1=0;
  x2=x[0]+0.5*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=y[0]-0.5*height; if(y1<0) y1=0;
  y2=y[0]+0.5*height; if(y2>HEIGHT-1) y2=HEIGHT-1;
  maxval=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+j*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog2.5x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog2.5y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[4]=xmax;
  ya[4]=ymax;

  for(i=1;i<=3;i++){
    d=sqrt((xa[4]-xa[i])*(xa[4]-xa[i])+(ya[4]-ya[i])*(ya[4]-ya[i]));
    if(d<MINSEP){fprintf(stderr,"error cog2.5.%d; separation %f\n",i,d); return 1;}
  }
 
  xa[0]=0; ya[0]=0;
  for(i=1;i<=4;i++){
    xa[0]+=xa[i]; ya[0]+=ya[i];
  }
  xa[0]/=4; ya[0]/=4;

  d1=sqrt((xa[1]-xa[2])*(xa[1]-xa[2])+(ya[1]-ya[2])*(ya[1]-ya[2]));
  d2=sqrt((xa[3]-xa[4])*(xa[3]-xa[4])+(ya[3]-ya[4])*(ya[3]-ya[3]));
  costheta=((xa[1]-xa[2])*(xa[3]-xa[4])+(ya[1]-ya[2])*(ya[3]-ya[4]))/d1/d2;
  degree=acos(costheta)/3.1415926*180;
  /*  fprintf(stderr,"\n\ncostheta=%f degree=%f\n\n",costheta,degree);*/
  if(degree < 80.0  || degree > 100.0 ){fprintf(stderr,"error cog2.6a crossing degree=%f\n",degree); return 1;}

  degree=abs(atan((ya[1]-ya[2])/(xa[1]-xa[2]))/3.1415926*180); 
  /*  fprintf(stderr,"\n\ncostheta=%f degree=%f\n\n",costheta,degree);*/
  if(degree < 60.0 ){fprintf(stderr,"error cog2.6b degree of pair 1=%f\n",degree); return 1;}
  

  fprintf(stderr,"%d.%06d\t",cputime.tv_sec,cputime.tv_usec);
  fprintf(stderr,"MODE2 : av=%e\t cog=(%f, %f)\r",av,xa[0],ya[0]);
  return 0;
}

int calccog3slowinterlace(int *pix, int width, int height, float *xa, float *ya, int line){
  /* line should be 0 or 1*/
  int i,j,h,k;
  float av;
  float sum;
  double d;
  int lx, ly;
  int x0,y0,x1,x2,y1,y2;
  int xmax,ymax;
  int maxval;
  int npix;
  int lpix;
  struct timeval cputime;
  struct timezone cpuzone;
  time_t now;
  struct tm *now_utc;
  int dx[5],dy[5];

  gettimeofday(&cputime, &cpuzone); 
  npix=width*height;

  /*Get UTC*/
  now=time(NULL);
  now_utc=gmtime(&now);

  /*Get CPU time*/
  gettimeofday(&cputime, &cpuzone); 
  npix=width*height;

  
  for(i=1;i<5;i++){
    dx[i]=xa[0]-xa[i];dy[i]=ya[0]-ya[i];
  }

  x0=xa[0]; y0=ya[0];
  for(i=0;i<5;i++){
    xa[i]=0;ya[i]=0;
  }

  /* Calculate Background */
  av=calcbackground(pix,xa[0],ya[0],width, height);
  
  /* Calculate COG */
  /*  x1=x0-0.5*width;x2=x0+0.5*width;
      y1=y0-0.5*height;y2=y0+0.5*height;*/
  x1=x0-0.5*width; if(x1<0) x1=0;
  x2=x0+0.5*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=y0-0.5*height; if(y1<0) y1=0;
  y2=y0+0.5*height; if(y2>HEIGHT-1) y2=HEIGHT-1;
  npix=0;
  sum=0;
  maxval=0;
  for(j=y1/2;j<y2/2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+(j*2+line)*WIDTH]-av;
      xa[0]+=(float) i*lpix;
      ya[0]+=(float) (j*2+line)*lpix;
      sum+=lpix;
      npix++;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j*2+line;
      }
    }
  }
  if(sum!=0){
    xa[0]/=sum;ya[0]/=sum;
  }
  else{
    xa[0]=0;ya[0]=0;
  }

  if(xa[0]>x2 || xa[0]<x1){ fprintf(stderr,"error inter%d-cog3s.1x xa[0]=%d x1=%d x2=%d\n",line,xa[0], x1, x2); return 1;}
  if(ya[0]>y2 || ya[0]<y1){ fprintf(stderr,"error inter%d-cog3s.1y ya[0]=%d y1=%d y2=%d\n",line,ya[0], y1, y2); return 1;}
  if(xmax > x2 || xmax < x1 ){ fprintf(stderr,"error inter%d-cog3s.2x xmax=%d x1=%d x2=%d\n", line,xmax, x1, x2); return 1;}
  if(ymax > y2 || ymax < y1 ){ fprintf(stderr,"error inter%d-cog3s.2y ymax=%d y1=%d y2=%d\n", line,ymax, y1, y2); return 1;}

  /* ADD offset to expected position of the stars*/
  for(i=1;i<=4;i++){
    xa[i]+=xa[0]-dx[i]; ya[i]+=ya[0]-dy[i];
  }

  d=(double) ((xa[0]-xa[2])*(xa[0]-xa[2])+(ya[0]-ya[2])*(ya[0]-ya[2]));
  d=sqrt(d);
  /*  fprintf(stderr,"d=%f",d); */
  width=(int) d/2/1.1;
  height=(int) d/2/1.1;
    
  /* Calculate COG1-4 */
  for(h=1;h<5;h++){
    x0=xa[h];y0=ya[h];
    x1=x0-width; if(x1<0) x1=0;
    x2=x0+width; if(x2>WIDTH-1) x2=WIDTH-1;
    y1=y0-height; if(y1<0) y1=0;
    y2=y0+height; if(y2>HEIGHT-1) y2=HEIGHT-1;
    npix=0;
    sum=0;
    xa[h]=0.0;ya[h]=0.0;
    for(j=y1/2;j<y2/2+1;j++){
      for(i=x1;i<x2;i++){
	lpix=pix[i+(j*2+line)*WIDTH]-av;
	xa[h]+=(float) i*lpix;
	ya[h]+=(float) (j*2+line)*lpix;
	sum+=lpix;
	npix++;
      }
    }
    if(sum!=0){
      xa[h]/=sum;ya[h]/=sum;
    }
    else{
      xa[h]=0;ya[h]=0;
      fprintf(stderr,"\nerror inter%d-cog3s.3.%d; sum=0 : npix=%d : av=%f x=[%d %d] y=[%d %d]\n",line,h,npix,av,x1,x2,y1,y2);
      return 1;
    }
    
    /* fprintf(stderr,"\tx[%d]=%f y[%d]=%f\n",i,i,xa[h],ya[h]); */
    if(xa[h] > x2 || xa[h] < x1 ){
      fprintf(stderr,"\nerror inter%d-cog3s.4x.%d; X out of range = %f should be between %d %d prev %d %d",line,h,xa[h],x1,x2,x0,width);
      return 1;
    }
    if(ya[h] > y2 || ya[h] < y1 ){
      fprintf(stderr,"\nerror inter%d-cog3s.4y.%d; Y out of range = %f should be between %d %d prev %f",line,h,ya[h],y1,y2,y0);
      return 1;
    }
  }

  xa[0]=0; ya[0]=0;
  for(i=1;i<=4;i++){
    xa[0]+=xa[i]; ya[0]+=ya[i];
  }
  xa[0]/=4; ya[0]/=4;

  fprintf(stdout,"%04d/%02d/%02d %02d:%02d:%02d.%06d\t",now_utc->tm_year+1900,now_utc->tm_mon+1,now_utc->tm_mday,now_utc->tm_hour,now_utc->tm_min,now_utc->tm_sec,cputime.tv_usec);
  fprintf(stdout,"MODE3SINTERLACE-%d : cog1= %f %f\tcog2= %f %f\tcog3= %f %f\tcog4= %f %f\n",line,xa[1]*SCLX,ya[1]*SCLY,xa[2]*SCLX,ya[2]*SCLY,xa[3]*SCLX,ya[3]*SCLY,xa[4]*SCLX,ya[4]*SCLY);
  return 0;

}

int calccog0slowinterlace(int *pix, int width, int height, float *xa, float *ya, int line){
  /* line should be 0 or 1*/
  int i,j,h,k;
  float av;
  float sum;
  double d;
  int lx, ly;
  int x0,y0,x1,x2,y1,y2;
  int xmax,ymax;
  int maxval;
  int npix;
  int lpix;
  struct timeval cputime;
  struct timezone cpuzone;
  time_t now;
  struct tm *now_utc;
  int dx[5],dy[5];

  gettimeofday(&cputime, &cpuzone); 

  /*Get UTC*/
  now=time(NULL);
  now_utc=gmtime(&now);

  /*Get CPU time*/
  gettimeofday(&cputime, &cpuzone); 

  npix=WIDTH*HEIGHT;
  /*npix=width*height; */

  
  for(i=1;i<5;i++){
    dx[i]=xa[0]-xa[i];dy[i]=ya[0]-ya[i];
  }

  x0=xa[0]; y0=ya[0];
  xa[0]=0;ya[0]=0;


  /* Calculate total COG (calccog1) */
  /* Calculate Background */
  av=calcbackground(pix,x0,y0,width, height);

  /* Calculate COG of full image */
  sum=0;
  for(j=0;j<HEIGHT/2;j++){
    for(i=0;i<WIDTH;i++){
      lpix=pix[i+(j*2+line)*WIDTH]-av;
      xa[0]+=(float) i*lpix;
      ya[0]+=(float) (j*2+line)*lpix;
      sum+=lpix;
    }
  }

  if(sum!=0){
    xa[0]/=sum;ya[0]/=sum;
  }
  else{
    xa[0]=0;ya[0]=0;
  }

  if(xa[0]< 1 || xa[0]> (WIDTH-1) || ya[0]< 1 || ya[0]> (HEIGHT-1)){
    fprintf(stderr,"error inter%d-cog0-1 xa[0]=%d ya[0]=%d av=%f\n",line,xa[0], ya[0], av);
    return 1;
  }

  x0=xa[0]; y0=ya[0];
  for(i=1;i<5;i++){
    xa[i]=0;ya[i]=0;
  }
  /* ADD offset to expected position of the stars*/
  for(i=1;i<=4;i++){
    xa[i]+=xa[0]-dx[i]; ya[i]+=ya[0]-dy[i];
  }

  d=(double) ((xa[0]-xa[2])*(xa[0]-xa[2])+(ya[0]-ya[2])*(ya[0]-ya[2]));
  d=sqrt(d);
  width=(int) d/2/1.1;
  height=(int) d/2/1.1;
    
  /* Calculate COG1-4 */
  for(h=1;h<5;h++){
    x0=xa[h];y0=ya[h];
    x1=x0-width; if(x1<0) x1=0;
    x2=x0+width; if(x2>WIDTH-1) x2=WIDTH-1;
    y1=y0-height; if(y1<0) y1=0;
    y2=y0+height; if(y2>HEIGHT-1) y2=HEIGHT-1;
    npix=0;
    sum=0;
    xa[h]=0.0;ya[h]=0.0;
    for(j=y1/2;j<y2/2+1;j++){
      for(i=x1;i<x2;i++){
	lpix=pix[i+(j*2+line)*WIDTH]-av;
	xa[h]+=(float) i*lpix;
	ya[h]+=(float) (j*2+line)*lpix;
	sum+=lpix;
	npix++;
      }
    }
    if(sum!=0){
      xa[h]/=sum;ya[h]/=sum;
    }
    else{
      xa[h]=0;ya[h]=0;
      fprintf(stderr,"\nerror inter%d-cog0s.3.%d; sum=0 : npix=%d : av=%f x=[%d %d] y=[%d %d]\n",line,h,npix,av,x1,x2,y1,y2);
      return 1;
    }
    
    if(xa[h] > x2 || xa[h] < x1 ){
      fprintf(stderr,"\nerror inter%d-cog0s.4x.%d; X out of range = %f should be between %d %d prev %d %d\n",line,h,xa[h],x1,x2,x0,width);
      return 1;
    }
    if(ya[h] > y2 || ya[h] < y1 ){
      fprintf(stderr,"\nerror inter%d-cog0s.4y.%d; Y out of range = %f should be between %d %d prev %f\n",line,h,ya[h],y1,y2,y0);
      return 1;
    }
  }

  xa[0]=0; ya[0]=0;
  for(i=1;i<=4;i++){
    xa[0]+=xa[i]; ya[0]+=ya[i];
  }
  xa[0]/=4; ya[0]/=4;

  fprintf(stdout,"%04d/%02d/%02d %02d:%02d:%02d.%06d\t",now_utc->tm_year+1900,now_utc->tm_mon+1,now_utc->tm_mday,now_utc->tm_hour,now_utc->tm_min,now_utc->tm_sec,cputime.tv_usec);
  fprintf(stdout,"MODE3SINTERLACE-%d : cog1= %f %f\tcog2= %f %f\tcog3= %f %f\tcog4= %f %f\n",line,xa[1]*SCLX,ya[1]*SCLY,xa[2]*SCLX,ya[2]*SCLY,xa[3]*SCLX,ya[3]*SCLY,xa[4]*SCLX,ya[4]*SCLY);
  return 0;

}

int calccog2slowinterlace(int *pix, int width, int height, float *xa, float *ya, int line){
  int i,j;
  float av;
  float sum;
  float d;
  int lx, ly;
  int x0,y0,x1,x2,y1,y2;
  int mx1,mx2,my1,my2;
  int xmax,ymax;
  int maxval;
  int npix;
  int lpix;
  float d1,d2,costheta,degree;
  struct timeval cputime;
  struct timezone cpuzone;

  gettimeofday(&cputime, &cpuzone); 
  npix=width*height;

  x0=xa[0]; y0=ya[0];
  for(i=1;i<5;i++){
    xa[i]=0;ya[i]=0;
  }


  /* Detect Peak 1 --- the Highest Peak in the 1-4th quadrant */
  x1=x0-0.25*width; if(x1<0) x1=0;
  x2=x0+0.25*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=y0-0.5*height; if(y1<0) y1=0;
  y2=y0-0.05*height; if(y2>HEIGHT-1) y2=HEIGHT-1;

  xmax=0;
  ymax=0;
  maxval=0;
  for(j=y1/2;j<y2/2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+(j*2+line)*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j*2+line;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog12.2x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog12.2y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[1]=xmax;
  ya[1]=ymax;

  /* Detect Peak 2 -- crossing of xa[1] */
  mx1=xa[1]-MASKWIDTH; if(mx1<0) mx1=0;
  mx2=xa[1]+MASKWIDTH; if(mx2>WIDTH-1) mx2=WIDTH-1;
  my1=ya[1]-MASKWIDTH; if(my1<0) my1=0;
  my2=ya[1]+MASKWIDTH; if(my2>HEIGHT-1) my2=HEIGHT-1;

  for(j=my1/2;j<my2/2;j++){
    for(i=mx1;i<mx2;i++){
      pix[i+(j*2+line)*WIDTH]=av;
    }
  }
  maxval=0;

  x1=2*xa[0]-xa[1]-MASKWIDTH; if(x1<0) x1=0;
  x2=2*xa[0]-xa[1]+MASKWIDTH; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=2*ya[0]-ya[1]-MASKWIDTH; if(y1<0) y1=0;
  y2=2*ya[0]-ya[1]+MASKWIDTH; if(y2>HEIGHT-1) y2=HEIGHT-1;

  for(j=y1/2;j<y2/2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+(j*2+line)*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j*2+line;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog12.3x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog12.3y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[2]=xmax;
  ya[2]=ymax;

  /* Detect Peak 3*/
  x1=x0-0.5*width; if(x1<0) x1=0;
  x2=x0+0.5*width; if(x2>WIDTH-1) x2=WIDTH-1;
  y1=y0-0.5*height; if(y1<0) y1=0;
  y2=y0+0.5*height; if(y2>HEIGHT-1) y2=HEIGHT-1;

  mx1=xmax-MASKWIDTH; if(mx1<0) x1=0;
  mx2=xmax+MASKWIDTH; if(mx2>WIDTH-1) x2=WIDTH-1;
  my1=ymax-MASKWIDTH; if(my1<0) y1=0;
  my2=ymax+MASKWIDTH; if(my2>HEIGHT-1) y2=HEIGHT-1;

  for(j=my1/2;j<my2/2;j++){
    for(i=mx1;i<mx2;i++){
      pix[i+(j*2+line)*WIDTH]=av;
    }
  }

  maxval=0;
  for(j=y1/2;j<y2/2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+(j*2+line)*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j*2+line;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog12.4x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog12.4y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[3]=xmax;
  ya[3]=ymax;
  
  for(i=1;i<=2;i++){
    d=sqrt((xa[3]-xa[i])*(xa[3]-xa[i])+(ya[3]-ya[i])*(ya[3]-ya[i]));
    if(d<MINSEP){fprintf(stderr,"error cog12.4.%d; separation %f\n",i,d); return 1;}
  }

  /* Detect Peak 4 */
  mx1=xmax-MASKWIDTH; if(mx1<0) x1=0;
  mx2=xmax+MASKWIDTH; if(mx2>WIDTH-1) x2=WIDTH-1;
  my1=ymax-MASKWIDTH; if(my1<0) y1=0;
  my2=ymax+MASKWIDTH; if(my2>HEIGHT-1) y2=HEIGHT-1;

  for(j=my1/2;j<my2/2;j++){
    for(i=mx1;i<mx2;i++){
      pix[i+(j*2+line)*WIDTH]=av;
    }
  }
  maxval=0;
  for(j=y1/2;j<y2/2;j++){
    for(i=x1;i<x2;i++){
      lpix=pix[i+(j*2+line)*WIDTH]-av;
      if(maxval<lpix){
	maxval=lpix;
	xmax=i;
	ymax=j*2+line;
      }
    }
  }
  if(xmax > x2 || xmax < x1 ){fprintf(stderr,"error cog12.5x xmax=%d x1=%d x2=%d\n",xmax,x1,x2); return 1;}
  if(ymax > y2 || ymax < y1 ){fprintf(stderr,"error cog12.5y ymax=%d y1=%d y2=%d\n",ymax,y1,y2); return 1;}
  xa[4]=xmax;
  ya[4]=ymax;

  for(i=1;i<=3;i++){
    d=sqrt((xa[4]-xa[i])*(xa[4]-xa[i])+(ya[4]-ya[i])*(ya[4]-ya[i]));
    if(d<MINSEP){fprintf(stderr,"error cog12.5.%d; separation %f\n",i,d); return 1;}
  }
 
  xa[0]=0; ya[0]=0;
  for(i=1;i<=4;i++){
    xa[0]+=xa[i]; ya[0]+=ya[i];
  }
  xa[0]/=4; ya[0]/=4;

  d1=sqrt((xa[1]-xa[2])*(xa[1]-xa[2])+(ya[1]-ya[2])*(ya[1]-ya[2]));
  d2=sqrt((xa[3]-xa[4])*(xa[3]-xa[4])+(ya[3]-ya[4])*(ya[3]-ya[3]));
  costheta=((xa[1]-xa[2])*(xa[3]-xa[4])+(ya[1]-ya[2])*(ya[3]-ya[4]))/d1/d2;
  degree=acos(costheta)/3.1415926*180;

  if(degree < 80.0  || degree > 100.0 ){fprintf(stderr,"error cog12.6a crossing degree=%f\n",degree); return 1;}

  degree=abs(atan((ya[1]-ya[2])/(xa[1]-xa[2]))/3.1415926*180); 

  if(degree < 60.0 ){fprintf(stderr,"error cog12.6b degree of pair 1=%f\n",degree); return 1;}
  

  fprintf(stderr,"%d.%06d\t",cputime.tv_sec,cputime.tv_usec);
  fprintf(stderr,"MODE2slow : av=%e\t cog=(%f, %f)\r",av,xa[0],ya[0]);
  return 0;
}

void calccogtot(int *pix){
  u_int *pixini;
  int cogstatus, cogstatus0, cogstatus1;
  int i,j,k;
  double dxa,dya,dxb,dyb;
  double dla,dta,dla2,dta2,dlb,dtb,dlb2,dtb2;
  double sigmala,sigmata,sigmalb,sigmatb;
  double seeingla,seeingta,seeinglb,seeingtb;
  double inter0seeingla,inter0seeingta,inter0seeinglb,inter0seeingtb;
  double inter1seeingla,inter1seeingta,inter1seeinglb,inter1seeingtb;

  FILE *seeingdat;

  struct timeval cputime; 
  struct timezone cpuzone; 

  if(flag==2){
    for(i=0;i<5;i++){
      inter0x[i]=x[i]; inter0y[i]=y[i];
      inter1x[i]=x[i]; inter1y[i]=y[i];
    }
    if(fflag==0){
      /* MODE for FAST measurement */
      cogstatus0=calccog3slowinterlace(pix, CALCWIDTH, CALCHEIGHT, inter0x, inter0y, 0);  
      cogstatus1=calccog3slowinterlace(pix, CALCWIDTH, CALCHEIGHT, inter1x, inter1y, 1);
      if(cogstatus0==1 || cogstatus1==1){
	if(nerror>MAXERROR){
	  fprintf(stderr,"\n");
	  pflag=flag;
	  flag=1;
	  nerror=0;
	}
	else{
	  nerror++;
	  /* If measurement of COGs failed, reload from the master array */
	  for(i=0;i<5;i++){
	    inter0x[i]=x[i]; inter0y[i]=y[i];
	    inter1x[i]=x[i]; inter1y[i]=y[i];
	  }
	}
      }
      /* If COGs are successfully measured, write it into master array */
      /* Write it into sigma array, too */
      else{
	for(i=0;i<5;i++){
	  x[i]=inter0x[i]; y[i]=inter0y[i];
	}

	inter0xfsig1[sigmacounter]=inter0x[1]*SCLX;inter0yfsig1[sigmacounter]=inter0y[1]*SCLY;
	inter0xfsig2[sigmacounter]=inter0x[2]*SCLX;inter0yfsig2[sigmacounter]=inter0y[2]*SCLY;
	inter0xfsig3[sigmacounter]=inter0x[3]*SCLX;inter0yfsig3[sigmacounter]=inter0y[3]*SCLY;
	inter0xfsig4[sigmacounter]=inter0x[4]*SCLX;inter0yfsig4[sigmacounter]=inter0y[4]*SCLY;
	
	inter1xfsig1[sigmacounter]=inter1x[1]*SCLX;inter1yfsig1[sigmacounter]=inter1y[1]*SCLY;
	inter1xfsig2[sigmacounter]=inter1x[2]*SCLX;inter1yfsig2[sigmacounter]=inter1y[2]*SCLY;
	inter1xfsig3[sigmacounter]=inter1x[3]*SCLX;inter1yfsig3[sigmacounter]=inter1y[3]*SCLY;
	inter1xfsig4[sigmacounter]=inter1x[4]*SCLX;inter1yfsig4[sigmacounter]=inter1y[4]*SCLY;
	sigmacounter++;
      }
    }
    else{
      /* MODE for SLOW measurement */
      cogstatus0=calccog0slowinterlace(pix, CALCWIDTH, CALCHEIGHT, inter0x, inter0y, 0);  
      cogstatus1=calccog0slowinterlace(pix, CALCWIDTH, CALCHEIGHT, inter1x, inter1y, 1);
      if(cogstatus0==1 || cogstatus1==1){
        if(nerror>MAXERROR){
	  fprintf(stderr,"\n");
	  pflag=flag;
	  flag=0;
	  nerror=0;
	}
	else{
	  nerror++;
	  /* If measurement of COGs failed, reload from the master array */
	  for(i=0;i<5;i++){
	    inter0x[i]=x[i]; inter0y[i]=y[i];
	    inter1x[i]=x[i]; inter1y[i]=y[i];
	  }
	}
      }
      /* If COGs are successfully measured, write it into the master array */
      else{
	for(i=0;i<5;i++){
	  x[i]=inter0x[i]; y[i]=inter0y[i];
	}
	inter0xfsig1[sigmacounter]=inter0x[1]*SCLX;inter0yfsig1[sigmacounter]=inter0y[1]*SCLY;
	inter0xfsig2[sigmacounter]=inter0x[2]*SCLX;inter0yfsig2[sigmacounter]=inter0y[2]*SCLY;
	inter0xfsig3[sigmacounter]=inter0x[3]*SCLX;inter0yfsig3[sigmacounter]=inter0y[3]*SCLY;
	inter0xfsig4[sigmacounter]=inter0x[4]*SCLX;inter0yfsig4[sigmacounter]=inter0y[4]*SCLY;
	
	inter1xfsig1[sigmacounter]=inter1x[1]*SCLX;inter1yfsig1[sigmacounter]=inter1y[1]*SCLY;
	inter1xfsig2[sigmacounter]=inter1x[2]*SCLX;inter1yfsig2[sigmacounter]=inter1y[2]*SCLY;
	inter1xfsig3[sigmacounter]=inter1x[3]*SCLX;inter1yfsig3[sigmacounter]=inter1y[3]*SCLY;
	inter1xfsig4[sigmacounter]=inter1x[4]*SCLX;inter1yfsig4[sigmacounter]=inter1y[4]*SCLY;
	sigmacounter++;
      }
    }

    /* Calculate sigma for each sequential MAXSIGMA data points */

    if(sigmacounter>=MAXSIGMA){
      if(pflag!=2) fprintf(stderr,"\n");
      pflag=flag;
      fprintf(stdout,"# %d frames acquired in sequence\n",MAXSIGMA);

      /* seeing using odd lines*/
      dla=0;dla2=0;dta=0;dta2=0;
      dlb=0;dlb2=0;dtb=0;dtb2=0;
      for(i=0;i<MAXSIGMA;i++){
	dxa=inter0xfsig1[i]-inter0xfsig2[i];
	dya=inter0yfsig1[i]-inter0yfsig2[i];
	dxb=inter0xfsig3[i]-inter0xfsig4[i];
	dyb=inter0yfsig3[i]-inter0yfsig4[i];

	dla+=dya/MAXSIGMA;
	dla2+=dya*dya/MAXSIGMA;
	dta+=dxa/MAXSIGMA;
	dta2+=dxa*dxa/MAXSIGMA;
	dlb+=dxb/MAXSIGMA;
	dlb2+=dxb*dxb/MAXSIGMA;
	dtb+=dyb/MAXSIGMA;
	dtb2+=dyb*dyb/MAXSIGMA;
      }
      sigmala=sqrt(dla2-dla*dla);sigmata=sqrt(dta2-dta*dta);
      sigmalb=sqrt(dlb2-dlb*dlb);sigmatb=sqrt(dtb2-dtb*dtb);
      sigmala*=(M_PI/180/3600);
      sigmata*=(M_PI/180/3600);
      sigmalb*=(M_PI/180/3600);
      sigmatb*=(M_PI/180/3600);

      inter0seeingla=pow(2,1.4)/M_PI*pow(sigmala,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.0968*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;
      inter0seeingta=pow(2,1.4)/M_PI*pow(sigmata,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.145*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;
      inter0seeinglb=pow(2,1.4)/M_PI*pow(sigmalb,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.0968*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;
      inter0seeingtb=pow(2,1.4)/M_PI*pow(sigmatb,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.145*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;


      /* seeing using even lines*/
      dla=0;dla2=0;dta=0;dta2=0;
      dlb=0;dlb2=0;dtb=0;dtb2=0;
      for(i=0;i<MAXSIGMA;i++){
	dxa=inter1xfsig1[i]-inter1xfsig2[i];
	dya=inter1yfsig1[i]-inter1yfsig2[i];
	dxb=inter1xfsig3[i]-inter1xfsig4[i];
	dyb=inter1yfsig3[i]-inter1yfsig4[i];

	dla+=dya/MAXSIGMA;
	dla2+=dya*dya/MAXSIGMA;
	dta+=dxa/MAXSIGMA;
	dta2+=dxa*dxa/MAXSIGMA;
	dlb+=dxb/MAXSIGMA;
	dlb2+=dxb*dxb/MAXSIGMA;
	dtb+=dyb/MAXSIGMA;
	dtb2+=dyb*dyb/MAXSIGMA;
      }
      sigmala=sqrt(dla2-dla*dla);sigmata=sqrt(dta2-dta*dta);
      sigmalb=sqrt(dlb2-dlb*dlb);sigmatb=sqrt(dtb2-dtb*dtb);
      sigmala*=(M_PI/180/3600);
      sigmata*=(M_PI/180/3600);
      sigmalb*=(M_PI/180/3600);
      sigmatb*=(M_PI/180/3600);

      inter1seeingla=pow(2,1.4)/M_PI*pow(sigmala,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.0968*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;
      inter1seeingta=pow(2,1.4)/M_PI*pow(sigmata,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.145*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;
      inter1seeinglb=pow(2,1.4)/M_PI*pow(sigmalb,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.0968*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;
      inter1seeingtb=pow(2,1.4)/M_PI*pow(sigmatb,1.2)/pow(LAMBDA,0.2)/pow((0.179*pow(DD,-0.333333333333333)-0.145*pow(D,-0.333333333333333)),0.6)*180*3600/4.0*0.98;

      seeingla=(inter0seeingla+inter1seeingla)/2;
      seeingta=(inter0seeingta+inter1seeingta)/2;
      seeinglb=(inter0seeinglb+inter1seeinglb)/2;
      seeingtb=(inter0seeingtb+inter1seeingtb)/2;


      fprintf(stderr,"Seeing A=\033[031m%f, %f\033[m\tB=\033[034m%f, %f\r\033[m",seeingla,seeingta,seeinglb,seeingtb);
      fprintf(stderr,"Seeing A=\033[031m%f, %f\033[m\tB=\033[034m%f, %f\r\033[m",seeingla,seeingta,seeinglb,seeingtb);
      fprintf(stdout,"# Seeing A=%f, %f\n",seeingla,seeingta);
      fprintf(stdout,"# Seeing B=%f, %f\n",seeinglb,seeingtb);
      fprintf(stdout,"# inter-0 Seeing A=%f, %f\n",inter0seeingla,inter0seeingta);
      fprintf(stdout,"# inter-0 Seeing B=%f, %f\n",inter0seeinglb,inter0seeingtb);
      fprintf(stdout,"# inter-1 Seeing A=%f, %f\n",inter1seeingla,inter1seeingta);
      fprintf(stdout,"# inter-1 Seeing B=%f, %f\n",inter1seeinglb,inter1seeingtb);

      sigmacounter=0;
      nerror=0;

      gettimeofday(&cputime, &cpuzone);
      sprintf(ccpusec,"%d.%06d",cputime.tv_sec,cputime.tv_usec);
      cpusec=atof(ccpusec)-cpusec0;

      /* Plot in GNUPLOT*/
      if((seeingdat=fopen(tmpseeingdata,"a+"))==NULL){
	fprintf(stderr,"Error : Failed to open %s",tmpseeingdata);
	exit(1);
      }
      
      fprintf(seeingdat,"%e\t%e %e %e %e %e %e %e %e %e %e %e %e\n",cpusec,seeingla,seeingta,seeinglb,seeingtb,inter0seeingla,inter0seeingta,inter0seeinglb,inter0seeingtb,inter1seeingla,inter1seeingta,inter1seeinglb,inter1seeingtb);
      fclose(seeingdat);

      if(gnuplotstatus==0){
	fprintf(gp, "set size 1,0.5\n");
	fprintf(gp, "set yrange [0:5]\n");
	fprintf(gp, "set xlabel \"Time(sec)\"\n");
	fprintf(gp, "set ylabel \"Seeing(arcsec)\"\n");
	fprintf(gp, "set grid\n");
	fprintf(gp, "set multiplot\n");
	fprintf(gp, "set origin 0,0.5\n");
	fprintf(gp, "plot \"%s\" using 1:2 title \"sigma_l\" with lines, ",tmpseeingdata);
	fprintf(gp, " \"%s\" using 1:3 title \"sigma_t\" with lines \n",tmpseeingdata);
	fprintf(gp, "set origin 0,0\n");
	fprintf(gp, "plot \"%s\" using 1:4 title \"sigma_l\" with lines, ",tmpseeingdata);
	fprintf(gp, " \"%s\" using 1:5 title \"sigma_t\" with lines \n",tmpseeingdata);
	fflush(gp);
      }
    
    }
  }
  else if(flag==1){
    cogstatus=calccog2(pix, CALCWIDTH, CALCHEIGHT, x, y);  
    if(cogstatus==1){
      /*fprintf(stderr,"\n");*/
      pflag=flag;
      flag=0;
    }
    if(cogstatus==0){
      /*fprintf(stderr,"\n");*/
      pflag=flag;
      flag=2;
    }
    sigmacounter=0;
  }
  else{
    if(fflag==0){
      cogstatus=calccog1(pix, x, y);  
      /*      fprintf(stderr,"calccogstatus1=%d",cogstatus);*/
      if(cogstatus==1){
	/*      fprintf(stderr,"\n"); */
	pflag=flag;
	flag=0;
      }
      if(cogstatus==0){
	fprintf(stderr,"\n");
	pflag=flag;
	flag=1;
      }
    }
    else{
      cogstatus=calccog1(pix, x, y);
      cogstatus=calccog2slowinterlace(pix, CALCWIDTH, CALCHEIGHT, x, y, 0);
      if(cogstatus==1){
	/*      fprintf(stderr,"\n"); */
	pflag=flag;
	flag=0;
      }
      if(cogstatus==0){
	fprintf(stderr,"\n");
	pflag=flag;
	flag=2;
      }
    }    
    sigmacounter=0;
  }
}

float calcbackground(int *pix, int x0, int y0, int width, int height){
  int i,j;
  float av,sum;
  int x1,x2,y1,y2;
  int npix, lpix;

  /* Calculate Background */
  av=1e6;
  sum=0;
  x1=x0-2*width;x2=x0-width;
  y1=y0-0.5*height;y2=y0+0.5*height;
  if(x1<0) x1=0;
  if(x2>=WIDTH) x2=WIDTH-1;
  if(y1<0) y1=0;
  if(y2>=HEIGHT) y2=HEIGHT-1;

  npix=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      sum+=pix[i+j*WIDTH];
      npix++;
    }
  }
  if(av>sum/npix && npix>0) av=sum/npix;

  x1=x0-0.5*width;x2=x0+0.5*width;
  y1=y0-2*height;y2=y0-height;
  if(x1<0) x1=0;
  if(x2>=WIDTH) x2=WIDTH-1;
  if(y1<0) y1=0;
  if(y2>=HEIGHT) y2=HEIGHT-1;

  npix=0;
  sum=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      sum+=pix[i+j*WIDTH];
      npix++;
    }
  }
  if(av>sum/npix && npix>0) av=sum/npix;

  x1=x0+width;x2=x0+2*width;
  y1=y0-0.5*height;y2=y0+0.5*height;
  if(x1<0) x1=0;
  if(x2>=WIDTH) x2=WIDTH-1;
  if(y1<0) y1=0;
  if(y2>=HEIGHT) y2=HEIGHT-1;

  npix=0;
  sum=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      sum+=pix[i+j*WIDTH];
      npix++;
    }
  }
  if(av>sum/npix && npix>0) av=sum/npix;

  x1=x0-0.5*width;x2=x0+0.5*width;
  y1=y0+height;y2=y0+2*height;
  if(x1<0) x1=0;
  if(x2>=WIDTH) x2=WIDTH-1;
  if(y1<0) y1=0;
  if(y2>=HEIGHT) y2=HEIGHT-1;

  npix=0;
  sum=0;
  for(j=y1;j<y2;j++){
    for(i=x1;i<x2;i++){
      sum+=pix[i+j*WIDTH];
      npix++;
    }
  }
  if(av>sum/npix && npix>0) av=sum/npix;

  return av;
}

void fitssave(u_int *pix, char savefile[]){
  fitsfile *fptr;
  int fiostatus, nfound, anynull;
  long fpixel, npixels;
  int bitpix=8;
  long naxis=2;

  long naxes[2]={WIDTH, HEIGHT};

  /*FITS ؤν񤭽Ф */
  fprintf(stderr,"%s\n",savefile);
  unlink(savefile);
  fiostatus=0;
  if (fits_create_file(&fptr, savefile, &fiostatus)){ 
    fits_report_error(stderr,fiostatus); 
    exit(1);
  } 
  if (fits_create_img(fptr, bitpix, naxis, naxes, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fpixel=1;
  if (fits_write_img(fptr, TUINT, fpixel, NPIX, pix, &fiostatus)) 
    fits_report_error(stderr,fiostatus);
  if (fits_close_file(fptr, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
}

void fitssave2(u_int *pix, char savefile[]){
  fitsfile *fptr;
  int fiostatus, nfound, anynull;
  long fpixel, npixels;
  int bitpix=8;
  long naxis=2;
  u_int *lpix1,*lpix2;
  int i,j;
  char savefile1[100],savefile2[100];

  long naxes[2]={WIDTH, HEIGHT/2};

  lpix1=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));
  lpix2=(u_int *) malloc(WIDTH*HEIGHT*sizeof(int));

  for(j=0;j<HEIGHT/2;j++)
    for(i=0;i<WIDTH;i++)
      lpix1[i+j*WIDTH]=pix[i+j*2*WIDTH];
  for(j=0;j<HEIGHT/2;j++)
    for(i=0;i<WIDTH;i++)
      lpix2[i+j*WIDTH]=pix[i+(j*2+1)*WIDTH];
  
  sprintf(savefile1,"a.%s",savefile);
  sprintf(savefile2,"b.%s",savefile);

  /*FITS ؤν񤭽Ф */
  unlink(savefile1);
  fiostatus=0;
  if (fits_create_file(&fptr, savefile1, &fiostatus)){ 
    fits_report_error(stderr,fiostatus); 
    exit(1);
  } 
  if (fits_create_img(fptr, bitpix, naxis, naxes, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fpixel=1;
  if (fits_write_img(fptr, TUINT, fpixel, NPIX, lpix1, &fiostatus)) 
    fits_report_error(stderr,fiostatus);
  if (fits_close_file(fptr, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fprintf(stderr,"Halfsize image saved as %s\n",savefile1);

  unlink(savefile2);
  fiostatus=0;
  if (fits_create_file(&fptr, savefile2, &fiostatus)){ 
    fits_report_error(stderr,fiostatus); 
    exit(1);
  } 
  if (fits_create_img(fptr, bitpix, naxis, naxes, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fpixel=1;
  if (fits_write_img(fptr, TUINT, fpixel, NPIX, lpix2, &fiostatus)) 
    fits_report_error(stderr,fiostatus);
  if (fits_close_file(fptr, &fiostatus)){
    fits_report_error(stderr,fiostatus);
    exit(1);
  }
  fprintf(stderr,"Halfsize image saved as %s\n",savefile2);

  free(lpix1);
  free(lpix2);
}

void glDrawCircle(int x, int y, float radius) 
{
  int i;
  int res=16;

  glBegin(GL_LINES);
  for (i=0;i<res;i++)
    glVertex2d(x + radius * cos((2.* M_PI * (double) i) / res),
	       y + radius * sin((2.* M_PI * (double) i) / res));
  glEnd();
}

/* MEADE Telescope Controller */
/* Implemented by Fumihiro Uraguchi at NAOJ */

int openSio()
{
  time_t now;
  struct tm *now_utc;
  struct termio tio,tin;
  int hour;
  char tx[15];

  if((siofd=open(FIFO_RS232C,O_RDWR | O_NOCTTY ,PERMS))<0){
    fprintf(stderr,"SIO device failure:\n");
    comstatus = 0;
    return -1;
  }
  
  ioctl(siofd,TCGETA, &tio);
  tin=tio;
  tio.c_cflag = B9600 | CS8 | CREAD | CLOCAL ;
  tio.c_iflag = 0;
  tio.c_oflag = 0;
  tio.c_lflag = 0;
  tio.c_cc[VMIN] =0;
  tio.c_cc[VTIME] =1;
  ioctl(siofd,TCSETA, &tio);
  fprintf(stderr,"\nSIO device ready:\n");
  comstatus = 1;

  /*Get UTC*/
  now=time(NULL);
  now_utc=gmtime(&now);

  hour = now_utc->tm_hour-10;
  if(hour<0) hour=hour+24;
  else if(hour>24) hour=hour-24;

  /* Synchronize Telescope Clock to PC clock*/
  sprintf(tx,":SL %02d:%02d:%02d#",hour,now_utc->tm_min,now_utc->tm_sec);
  /*  write(siofd,tx,15); */
  /* fprintf(stderr,"Sets the local time: %s\n", tx); */

  /*Get Az*/
  az = getAz();
  fprintf(stderr,"Azimuth %lf:\n", az);
  
  return 0;
}

int closeSio()
{
  sendCom(":Q#");
  sendCom(":Qn#");
  sendCom(":Qs#");
  sendCom(":Qw#");
  sendCom(":Qe#");
  close(siofd);
  fprintf(stderr,"SIO device closed:\n");
  comstatus = 0;
  
  return 0;
}

int sendCom(char *tx)
{
  if(!comstatus){
    fprintf(stderr,"SIO device not opened:\n");
    return -1;
  }
  write(siofd,tx,4);
  
  return 0;
}

int setSlewRate(unsigned int sp)
{
  int r;
  // 4 set slew rate to SLEW(max)
  // 3 set slew rate to FIND
  // 2 set slew rate to CNTR
  // 1 set slew rate to GUIDE
  
  if(!comstatus){
    fprintf(stderr,"SIO device not opened:\n");
    return -1;
  }
  
  if(sp==1){
    slewrate++;
    if(slewrate>4) slewrate=4;
  }
  else{
    slewrate--;
    if(slewrate<1) slewrate=1;
  }
  
  switch(slewrate){
  case 1:
    r = sendCom(":RG#");
    //fprintf(stderr,"Slew rate: Guide\n");
    break;
  case 2:
    r = sendCom(":RC#");
    //fprintf(stderr,"Slew rate: Center\n");
    break;
  case 3:
    r = sendCom(":RM#");
    //fprintf(stderr,"Slew rate: Find\n");
    break;
  case 4:
    r = sendCom(":RS#");
    //fprintf(stderr,"Slew rate: Slew\n");
    break;
  }
  
  if(r<0){
    fprintf(stderr,"Set slew rate failed:\n");
    return -1;
  }
  
  return 0;
}

int AutoGuide(void)
{
  float ex = 0.0;
  float ey = 0.0; /* Guide error - x,y */
  float en = 0.0;
  float ee = 0.0; /* Guide error - north, east */
  double fr = 0.0; /* Field rotation - cw:+ */

  agc++;
  if(agc>1800){
    az = getAz();
    agc = 0;
    if(az<0.0){
      fprintf(stderr,"Can not get Az:\n");
      return -1;
    }
  }
 
  if(!comstatus){
    fprintf(stderr,"SIO device not opened:\n");
    return -1;
  }
  sendCom(":RG#");slewrate=1;
  ex=x[0]-(float)tgx;ey=y[0]-(float)tgy;
  fr = M_PI/180.0*(270-az); /* Field Rotation [rad] */
  en = ex*cos(fr)+ey*sin(fr);
  ee = -1*ex*sin(fr)+ey*cos(fr);

  if((ee < nz)&&(ee > -1*nz)){
    geflag = 1;
    sendCom(":Qw#");sendCom(":Qe#");
    zslewstatus=0;
  }
  if((en < nz)&&(en > -1*nz)){
    gnflag = 1;
    sendCom(":Qs#");sendCom(":Qn#");
    aslewstatus=0;
  }
  if((ee > nz)||(ee < -1*nz)){
    if(geflag){
      geflag=0;
      zslewstatus=1;
      if(ee>0) sendCom(":Mw#");
      else sendCom(":Me#");
    }
  }
  if((en > nz)||(en < -1*nz)){
    if(gnflag){
      gnflag=0;
      aslewstatus=1;
      if(en>0) sendCom(":Ms#");
      else sendCom(":Mn#");
    }
  }
  return 0;
}

int setObj(int num)
{
  FILE *fp;
  char tx[15];
  char line[100];

  sprintf(tx,":LS %04d#",num);
  write(siofd,tx,15);
  fp=fopen(FIFO_RS232C,"r");
  sprintf(tx,":LI#");
  write(siofd,tx,4);
  fgets(line,sizeof(line),fp);
  line[strlen(line)-1]=0x00;
  fprintf(stderr,"%s\n",line);
  fclose(fp);
 
  return 0;
}

double getAlt(void)
{
  FILE *fp;
  char tx[15];
  char line[100];

  double talt;
  char sign[2];
  char dum[10];
  int deg;
  int min;
  int sec;

  if(comstatus){
    fp=fopen(FIFO_RS232C,"r");
    sprintf(tx,":GA#");
    write(siofd,tx,4);
    fgets(line,sizeof(line),fp);
    if(strlen(line)<7){
      sprintf(tx,":U#");
      write(siofd,tx,3);
      sprintf(tx,":GA#");
      write(siofd,tx,4);
      fgets(line,sizeof(line),fp);
    }
    line[strlen(line)-1]=0x00;
    sscanf(line,"%1s%2d%1s%2d%1s%2d",&sign,&deg,&dum,&min,&dum,&sec);
    if(!strcmp(sign,"-")) talt = -1.0*(deg+min/60.0+sec/3600.0);
    else talt = deg+min/60.0+sec/3600.0;
    fclose(fp);
 
    return talt;
  }

  return 0.0;
}

double getAz(void)
{
  FILE *fp;
  char tx[15];
  char line[100];

  double taz;
  char dum[6];
  unsigned int d = 0;
  unsigned int m = 0;
  unsigned int s = 0;

  if(comstatus){
    fp=fopen(FIFO_RS232C,"r");
    sprintf(tx,":GZ#");
    write(siofd,tx,4);
    fgets(line,sizeof(line),fp);
    line[strlen(line)-1]=0x00;
    if(strlen(line)<7){
      sscanf(line,"%3d%2s%2d",&d,&dum,&m);
      s = 0;
    }
    else{
      sscanf(line,"%3d%2s%2d%1s%2d",&d,&dum,&m,&dum,&s);
    }
    taz = d+m/60.0+s/3600.0;
    fclose(fp);
    return taz;
  }

  return -1.0;
}

